@@ -39,9 +39,6 @@
 #include "warpmanager.h"
 #include "properties.h"
 #include "board.h"
-#ifdef GGZ_CLIENT
-#include "ggz-network.h"
-#endif
 
 #include "worm.h"
 
@@ -111,16 +108,7 @@ worm_set_direction (int worm, int dir)
 void
 worm_handle_direction (int worm, int dir)
 {
-  if (ggz_network_mode) {
-#ifdef GGZ_CLIENT
-    network_game_move (dir);
-
-    worms[0]->direction = dir;
-    worms[0]->keypress = 1;
-#endif
-  } else {
-    worm_set_direction (worm, dir);
-  }
+  worm_set_direction (worm, dir);
 }
 
 static void
@@ -166,9 +154,12 @@ static void
 gnibbles_worm_add_actor (GnibblesWorm *worm)
 {
   ClutterActor *actor;
+  GError *error = NULL;
 
-  actor = gtk_clutter_texture_new_from_pixbuf (
-            worm_pixmaps[properties->wormprops[worm->number]->color - 12]);
+  actor = gtk_clutter_texture_new ();
+  gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (actor),
+                                       worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                       &error);
   clutter_actor_set_size (actor, properties->tilesize, properties->tilesize);
   clutter_actor_set_position (actor,
                               worm->xhead * properties->tilesize,
@@ -198,7 +189,7 @@ gnibbles_worm_handle_keypress (GnibblesWorm * worm, guint keyval)
   if (worm->lives <= 0)
     return FALSE;
 
-  props = properties->wormprops[ggz_network_mode ? 0 : worm->number];
+  props = properties->wormprops[worm->number];
   propsUp = toupper(props->up);
   propsLeft = toupper(props->left);
   propsDown = toupper(props->down);
@@ -273,10 +264,10 @@ gnibbles_worm_reverse (gpointer data)
   GnibblesWorm *worm = (GnibblesWorm *) data;
   gint tmp, old_dir;
 
-  worm->list = g_list_reverse (worm->list);
-
   old_dir = gnibbles_worm_get_tail_direction (worm);
 
+  worm->list = g_list_reverse (worm->list);
+
   tmp = worm->xhead;
   worm->xhead = worm->xtail;
   worm->xtail = tmp;
@@ -303,7 +294,7 @@ gnibbles_worm_grok_bonus (GnibblesWorm *worm)
   int i;
 
   if (gnibbles_boni_fake (boni, worm->xhead, worm->yhead)) {
-    g_timeout_add (1, (GtkFunction) gnibbles_worm_reverse, worm);
+    g_timeout_add (1, (GSourceFunc) gnibbles_worm_reverse, worm);
     games_sound_play ("reverse");
     return;
   }
@@ -481,13 +472,16 @@ gnibbles_worm_animate_death (GnibblesWorm *worm)
 {
   ClutterActor *group = clutter_group_new ();
   ClutterActor *tmp = NULL;
+  GError *error = NULL;
 
   int i;
   gfloat x,y;
 
   for (i = 0; i < g_list_length (worm->list); i++) {
-    tmp = gtk_clutter_texture_new_from_pixbuf (
-            worm_pixmaps [properties->wormprops[worm->number]->color - 12]);
+    tmp = gtk_clutter_texture_new ();
+    gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (tmp),
+                                         worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                         &error);
 
     clutter_actor_get_position (CLUTTER_ACTOR (g_list_nth_data (worm->list, i)),
                                 &x, &y);
@@ -661,7 +655,7 @@ gnibbles_worm_rescale (GnibblesWorm *worm, gint tilesize)
                                 (y_pos / properties->tilesize) * tilesize);
 
     gtk_clutter_texture_set_from_pixbuf (
-       CLUTTER_TEXTURE (tmp),
+       GTK_CLUTTER_TEXTURE (tmp),
        worm_pixmaps[properties->wormprops[worm->number]->color - 12],
        &err);
     if (err)
@@ -704,6 +698,7 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
   gfloat x,y;
   ClutterActor *tmp = NULL;
   ClutterActor *group = clutter_group_new ();
+  GError *error = NULL;
 
   if (erasesize) {
     if (g_list_length (worm->list) <= erasesize) {
@@ -712,8 +707,10 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
     }
 
     for (i = 0; i < erasesize; i++) {
-      tmp = gtk_clutter_texture_new_from_pixbuf (
-              worm_pixmaps[properties->wormprops[worm->number]->color - 12]);
+      tmp = gtk_clutter_texture_new ();
+      gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (tmp),
+                                           worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                           &error);
       clutter_actor_get_position
         (CLUTTER_ACTOR (g_list_last (worm->list)->data), &x, &y);
       clutter_actor_set_position (CLUTTER_ACTOR (tmp), x, y);
@@ -727,7 +724,7 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
     worm->length -= erasesize;
     clutter_container_add_actor (CLUTTER_CONTAINER (stage), group);
 
-    clutter_actor_animate (group, CLUTTER_EASE_OUT_ELASTIC, 850,
+    clutter_actor_animate (group, CLUTTER_EASE_OUT_EXPO, 850,
                            "opacity", 0,
                            NULL);
   }
