@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
-import gtk, gobject
+from gi.repository import Gtk,GdkPixbuf,GObject
 import os.path
 import sudoku, saver, sudoku_maker
 import sudoku_thumber
 from gettext import gettext as _
-from timer import format_time, format_friendly_date
+from gettext import ngettext
+import time
 import defaults
 from simple_debug import simple_debug
 from colors import color_hex_to_float
@@ -61,21 +62,25 @@ class NewOrSavedGameSelector:
         self.saved_games = None
 
     def setup_dialog (self):
-        builder = gtk.Builder()
+        builder = Gtk.Builder()
         builder.set_translation_domain(defaults.DOMAIN)
         builder.add_from_file(self.ui_file)
         self.dialog = builder.get_object('dialog1')
-        self.dialog.set_default_response(gtk.RESPONSE_CANCEL)
+        self.dialog.set_default_response(Gtk.ResponseType.CANCEL)
         self.dialog.connect('close', self.close)
         self.dialog.hide()
+        saved_game_frame = builder.get_object('savedGameFrame')
         saved_game_view = builder.get_object('savedGameIconView')
         saved_game_widgets = [
+            saved_game_frame,
             saved_game_view,
             builder.get_object('savedGameLabel')
             ]
         builder.get_object('savedGameLabel').set_mnemonic_widget(
             saved_game_view
             )
+        new_game_frame = builder.get_object('newGameFrame')
+        new_game_frame.show()
         new_game_view = builder.get_object('newGameIconView')
         builder.get_object('newGameLabel').set_mnemonic_widget(
             new_game_view
@@ -90,7 +95,8 @@ class NewOrSavedGameSelector:
             for w in saved_game_widgets:
                 w.hide()
         else:
-            self.saved_game_model.set_sort_column_id(2, gtk.SORT_DESCENDING)
+            saved_game_frame.show()
+            self.saved_game_model.set_sort_column_id(2, Gtk.SortType.DESCENDING)
             saved_game_view.set_model(self.saved_game_model)
             saved_game_view.set_markup_column(0)
             saved_game_view.set_pixbuf_column(1)
@@ -98,19 +104,24 @@ class NewOrSavedGameSelector:
             view.set_item_width(150)
             view.set_columns(4)
             view.set_spacing(12)
-            view.set_selection_mode(gtk.SELECTION_SINGLE)
+            view.set_selection_mode(Gtk.SelectionMode.SINGLE)
         saved_game_view.connect('item-activated', self.saved_item_activated_cb)
+        saved_game_view.connect('button-release-event', self.item_clicked_cb)
         new_game_view.connect('item-activated', self.new_item_activated_cb)
+        new_game_view.connect('button-release-event', self.item_clicked_cb)
 
     @simple_debug
     def make_new_game_model (self):
         # Description, Pixbuf, Puzzle (str)
-        self.new_game_model = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
+        self.new_game_model = Gtk.ListStore(str, GdkPixbuf.Pixbuf, str)
         saved_games_to_exclude = [
             g['game'].split('\n')[0] for g in self.saved_games
             ]
         for cat in sudoku.DifficultyRating.ordered_categories:
-            label = sudoku.DifficultyRating.label_by_cat[cat]
+            label = {'easy': _('Easy'),
+                     'medium': _('Medium'),
+                     'hard': _('Hard'),
+                     'very hard': _('Very hard')}[cat]
             puzzles = self.sudoku_maker.get_puzzles(1, [cat], new = True,
                                                     exclude = saved_games_to_exclude
                                                     )
@@ -138,19 +149,60 @@ class NewOrSavedGameSelector:
                                         puzzle
                                         ))
 
+    def format_friendly_date (self, tim):
+        local_tim = time.localtime(tim)
+        diff = int(time.time() - tim)
+        curr_hour, curr_min = time.localtime()[3:5]
+        now_to_yesterday = curr_hour * 60 * 60 + curr_min * 60
+        if diff < now_to_yesterday:
+            # Then we're today
+            if diff < 60: # within the minute
+                return ngettext("Last played %(n)s second ago",
+                                "Last played %(n)s seconds ago", diff) % {'n': diff}
+            elif diff < (60 * 60): # within the hour...
+                minute = int(diff / 60)
+                return ngettext("Last played %(n)s minute ago",
+                                "Last played %(n)s minutes ago", minute) % {'n': minute}
+            else:
+                # Translators, see strftime manual in order to translate %? format strings
+                return time.strftime(_("Last played at %I:%M %p"), local_tim)
+        elif diff < now_to_yesterday + (60 * 60 * 24):
+            # Translators, see strftime manual in order to translate %? format strings
+            return time.strftime(_("Last played yesterday at %I:%M %p"), local_tim)
+        elif diff < now_to_yesterday + (60 * 60 * 24) * 6:
+            # Translators, see strftime manual in order to translate %? format strings
+            return time.strftime(_("Last played on %A at %I:%M %p"), local_tim)
+        else:
+            # Translators, see strftime manual in order to translate %? format strings
+            return time.strftime(_("Last played on %B %e %Y"), local_tim)
+
     @simple_debug
     def make_saved_game_model (self):
         # Description, Image, Last-Access time (for sorting), Puzzle (jar)
-        self.saved_game_model = gtk.ListStore(str, gtk.gdk.Pixbuf, int, gobject.TYPE_PYOBJECT)
+        self.saved_game_model = Gtk.ListStore(str, GdkPixbuf.Pixbuf, int, GObject.TYPE_PYOBJECT)
         for g in self.saved_games:
             game = g['game'].split('\n')[0]
             grid = sudoku.sudoku_grid_from_string(game)
             sr = sudoku.SudokuRater(grid.grid)
             sdifficulty = sr.difficulty()
-            lastPlayedText = _("Last Played %(timeAgo)s") % {'timeAgo': format_friendly_date(g['saved_at'])}
-            levelText =  _("%(level)s puzzle") % {'level': sdifficulty.value_string()}
-            durationText = _("Played for %(duration)s") % {
-                    'duration': format_time(g['timer.active_time'], round_at = 15, friendly = True)}
+            lastPlayedText = self.format_friendly_date(g['saved_at'])
+            try:
+                levelText = {'easy': _('Easy puzzle'),
+                             'medium': _('Medium puzzle'),
+                             'hard': _('Hard puzzle'),
+                             'very hard': _('Very hard puzzle')}[sdifficulty.value_category()]
+            except KeyError:
+                levelText = sdifficulty.value_category()
+            tim = g['timer.active_time']
+            if tim >= 3600.0:
+                hours = int(tim / 3600)
+                durationText = ngettext("Played for %d hour", "Played for %d hours", hours) % hours
+            elif tim >= 60.0:
+                minutes = int(tim / 60)
+                durationText = ngettext("Played for %d minute", "Played for %d minutes", minutes) % minutes
+            else:
+                seconds = int(tim)
+                durationText = ngettext("Played for %d second", "Played for %d seconds", seconds) % seconds            
             desc = "<b><i>%s</i></b>\n<span size='small'><i>%s</i>\n<i>%s.</i></span>" % (
                 levelText,
                 lastPlayedText,
@@ -174,23 +226,42 @@ class NewOrSavedGameSelector:
     def saved_item_activated_cb (self, iconview, path):
         self.resume_game(iconview.get_model()[path][3])
 
+    @simple_debug
+    def item_clicked_cb (self, wdgt, evt):
+        '''Single click event handler
+
+        This callback activates the selected icon with a single click.
+        '''
+        if evt.button != 1:
+            return False
+        # See if the player clicked an icon
+        position = wdgt.get_path_at_pos(int(evt.x), int(evt.y))
+        if not position:
+            return False
+        if not wdgt.path_is_selected(position):
+            return False
+        # Select it, redraw it(strictly aesthetic), and then activate it
+        wdgt.select_path(position)
+        wdgt.get_parent_window().process_updates(False)
+        wdgt.item_activated(position)
+
     @simple_debug
     def resume_game (self, jar):
         self.puzzle = (self.SAVED_GAME, jar)
-        self.dialog.emit('response', gtk.RESPONSE_OK)
+        self.dialog.emit('response', Gtk.ResponseType.OK)
 
     @simple_debug
     def play_game (self, puzzle):
         self.puzzle = (self.NEW_GAME, puzzle)
-        self.dialog.emit('response', gtk.RESPONSE_OK)
+        self.dialog.emit('response', Gtk.ResponseType.OK)
 
     @simple_debug
     def close (self):
-        self.dialog.emit('response', gtk.RESPONSE_CLOSE)
+        self.dialog.emit('response', Gtk.ResponseType.CLOSE)
 
     @simple_debug
     def handle_response (self, response):
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             return self.puzzle
         else:
             return None
