@@ -501,7 +501,7 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
   g_signal_connect (G_OBJECT (self), "hide", 
 		      G_CALLBACK (games_scores_dialog_hide), NULL);
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))),
                     vbox, TRUE, TRUE, 0);
@@ -521,10 +521,10 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
 	                   GTK_JUSTIFY_CENTER);    
   gtk_box_pack_start (GTK_BOX (vbox), self->priv->message, FALSE, FALSE, 0);
 
-  self->priv->hdiv = gtk_hseparator_new ();
+  self->priv->hdiv = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
   gtk_box_pack_start (GTK_BOX (vbox), self->priv->hdiv, FALSE, FALSE, 0);
 
-  self->priv->catbar = gtk_hbox_new (FALSE, 12);
+  self->priv->catbar = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (vbox), self->priv->catbar, FALSE, FALSE, 0);
 
   self->priv->label = gtk_label_new (NULL);
